#include "parallel.h"

#include <thread>
#include <vector>
#include <mutex>
#include <condition_variable>

namespace parallel {

void runJobs(
		int jobs, const std::vector<void *> &elems,
		std::function<void(void *)> func) {
	int currentJobs = 0;
	std::mutex mut;
	std::unique_lock<std::mutex> lock(mut);
	std::condition_variable cond;

	for (void *elem: elems) {
		cond.wait(lock, [&] { return currentJobs < jobs; });

		currentJobs += 1;
		std::thread thread([elem, jobs, &currentJobs, &mut, &cond, &func] {
			func(elem);

			mut.lock();
			currentJobs -= 1;
			mut.unlock();
			cond.notify_one();
		});

		thread.detach();
	}

	cond.wait(lock, [&] { return currentJobs == 0; });
}

int coreCount() {
	return std::thread::hardware_concurrency();
}

}
