#pragma once

#include <functional>
#include <vector>

namespace parallel {

void runJobs(
		int jobs, const std::vector<void *> &elems,
		std::function<void(void *)> func);

template<typename Container, typename Func>
void parallel(int jobs, Container &cont, Func func) {
	if (jobs == 1) {
		for (auto &elem: cont) {
			func(elem);
		}
	} else {
		std::vector<void *> elems;
		for (auto &elem: cont) {
			elems.push_back((void *)&elem);
		}

		runJobs(jobs, elems, [&func](void *el) {
			func(*(typename Container::value_type *)el);
		});
	}
}

int coreCount();

}
